==> epoll-0.1.tar.gz: epoll/epollc.c <==
/* Copyright 2005 James Bunton <james@delx.cjb.net> */
/* Licensed for distribution under the GPL version 2, check COPYING for details */

#include <Python.h>
#include <sys/epoll.h>
#include <sys/errno.h>

static PyObject * meth_epoll_create(PyObject *self, PyObject *args)
{
	int size;
	int ret;

	if(!PyArg_ParseTuple(args, "i", &size))
		return NULL;
	ret = epoll_create(size);
	if(ret < 0)
		return Py_BuildValue("ii", ret, errno);
	else
		return Py_BuildValue("i", ret);
}

static PyObject * meth_epoll_ctl(PyObject *self, PyObject *args)
{
	int epfd;
	int op;
	int fd;
	int events;
	int ret;
	struct epoll_event ev;

	if(!PyArg_ParseTuple(args, "iiii", &epfd, &op, &fd, &events))
		return NULL;
	ev.data.fd = fd;
	ev.events = events;
	ret = epoll_ctl(epfd, op, fd, &ev);
	if(ret < 0)
		return Py_BuildValue("ii", ret, errno);
	else {
		Py_INCREF(Py_None);
		return Py_None;
	}
}

static PyObject * meth_epoll_wait(PyObject *self, PyObject *args)
{
	int epfd = 0;
	int maxevents = 0;
	int timeout = 0;
	int numEvents = 0;
	struct epoll_event *events = NULL;
	int i = 0;
	PyObject *retList = NULL;
	PyObject *val = NULL;

	if(!PyArg_ParseTuple(args, "iii", &epfd, &maxevents, &timeout))
		return NULL;
	events = calloc(maxevents, sizeof(struct epoll_event));
	if(events == NULL) {
		PyErr_NoMemory();
		return NULL;
	}
	numEvents = epoll_wait(epfd, events, maxevents, timeout);
	if(numEvents < 0) {
		free(events);
		return Py_BuildValue("ii", numEvents, errno);
	}
	
	retList = PyList_New(numEvents);
	if(retList == NULL) {
		free(events);
		return NULL;
	}
	
	for(i = 0; i < numEvents; i++) {
		val = Py_BuildValue("ii", events[i].data.fd, events[i].events);
		if(val == NULL) {
			free(events);
			Py_DECREF(retList);
			return NULL;
		}
		if(PyList_SetItem(retList, i, val) < 0) {
			free(events);
			Py_DECREF(val);
			Py_DECREF(retList);
			return NULL;
		}
	}
	free(events);
	return retList;
}

static PyMethodDef epollcMethods[] = {
	{"epoll_create", meth_epoll_create, METH_VARARGS, "Create an epoll file descriptor."},
	{"epoll_wait", meth_epoll_wait, METH_VARARGS, "Wait on an epoll file descriptor."},
	{"epoll_ctl", meth_epoll_ctl, METH_VARARGS, "Modify an epoll file descriptor."},

	{NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
initepollc(void)
{
	(void) Py_InitModule("epollc", epollcMethods);
}

==> epoll-0.2.tar.gz: epoll/epollc.c <==
/* Copyright 2005 James Bunton <james@delx.cjb.net> */
/* Licensed for distribution under the GPL version 2, check COPYING for details */

#include <Python.h>
#include <sys/epoll.h>
#include <sys/errno.h>

static PyObject * meth_epoll_create(PyObject *self, PyObject *args)
{
	int size;
	int ret;

	if(!PyArg_ParseTuple(args, "i", &size))
		return NULL;
	ret = epoll_create(size);
	if(ret < 0)
		return Py_BuildValue("ii", ret, errno);
	else
		return Py_BuildValue("i", ret);
}

static PyObject * meth_epoll_ctl(PyObject *self, PyObject *args)
{
	int epfd;
	int op;
	int fd;
	int events;
	int ret;
	struct epoll_event ev;

	if(!PyArg_ParseTuple(args, "iiii", &epfd, &op, &fd, &events))
		return NULL;
	ev.data.fd = fd;
	ev.events = events;
	ret = epoll_ctl(epfd, op, fd, &ev);
	if(ret < 0)
		return Py_BuildValue("ii", ret, errno);
	else {
		Py_INCREF(Py_None);
		return Py_None;
	}
}

static PyObject * meth_epoll_wait(PyObject *self, PyObject *args)
{
	int epfd = 0;
	int maxevents = 0;
	int timeout = 0;
	int numEvents = 0;
	struct epoll_event *events = NULL;
	int i = 0;
	PyObject *retList = NULL;
	PyObject *val = NULL;

	if(!PyArg_ParseTuple(args, "iii", &epfd, &maxevents, &timeout))
		return NULL;
	events = calloc(maxevents, sizeof(struct epoll_event));
	if(events == NULL) {
		PyErr_NoMemory();
		return NULL;
	}
	Py_BEGIN_ALLOW_THREADS;
	numEvents = epoll_wait(epfd, events, maxevents, timeout);
	Py_END_ALLOW_THREADS;
	if(numEvents < 0) {
		free(events);
		return Py_BuildValue("ii", numEvents, errno);
	}
	
	retList = PyList_New(numEvents);
	if(retList == NULL) {
		free(events);
		return NULL;
	}
	
	for(i = 0; i < numEvents; i++) {
		val = Py_BuildValue("ii", events[i].data.fd, events[i].events);
		if(val == NULL) {
			free(events);
			Py_DECREF(retList);
			return NULL;
		}
		if(PyList_SetItem(retList, i, val) < 0) {
			free(events);
			Py_DECREF(val);
			Py_DECREF(retList);
			return NULL;
		}
	}
	free(events);
	return retList;
}

static PyMethodDef epollcMethods[] = {
	{"epoll_create", meth_epoll_create, METH_VARARGS, "Create an epoll file descriptor."},
	{"epoll_wait", meth_epoll_wait, METH_VARARGS, "Wait on an epoll file descriptor."},
	{"epoll_ctl", meth_epoll_ctl, METH_VARARGS, "Modify an epoll file descriptor."},

	{NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
initepollc(void)
{
	PyObject *m;
	m = Py_InitModule("epollc", epollcMethods);
	PyModule_AddIntConstant(m, "EPOLLIN", EPOLLIN);
	PyModule_AddIntConstant(m, "EPOLLPRI", EPOLLPRI);
	PyModule_AddIntConstant(m, "EPOLLOUT", EPOLLOUT);
	PyModule_AddIntConstant(m, "EPOLLRDNORM", EPOLLRDNORM);
	PyModule_AddIntConstant(m, "EPOLLRDBAND", EPOLLRDBAND);
	PyModule_AddIntConstant(m, "EPOLLWRNORM", EPOLLWRNORM);
	PyModule_AddIntConstant(m, "EPOLLWRBAND", EPOLLWRBAND);
	PyModule_AddIntConstant(m, "EPOLLMSG", EPOLLMSG);
	PyModule_AddIntConstant(m, "EPOLLERR", EPOLLERR);
	PyModule_AddIntConstant(m, "EPOLLHUP", EPOLLHUP);
	PyModule_AddIntConstant(m, "EPOLLONESHOT", EPOLLONESHOT);
	PyModule_AddIntConstant(m, "EPOLLET", EPOLLET);
	PyModule_AddIntConstant(m, "EPOLL_CTL_ADD", EPOLL_CTL_ADD);
	PyModule_AddIntConstant(m, "EPOLL_CTL_DEL", EPOLL_CTL_DEL);
	PyModule_AddIntConstant(m, "EPOLL_CTL_MOD", EPOLL_CTL_MOD);
}
